"""django-temp-permissions app config."""

from django.apps import AppConfig


class Config(AppConfig):  # noqa D101
    verbose_name = "Django Temporary Permissions"
    name = "django_temp_permissions"
